function [final_t, final_Q, step_y, step_fitness, step_energy]=runSim_stability(Q0,stepLimit, M, I, L, R, g)
%[final_t, final_Q, step_y, step_fitness,
%           step_energy]=runSim_stability(Q0,stepLimit, M, I, L, R, g)
%
%  Modified version of the primary simulation function, monkeySim
% Limits simulation to specified # of steps, records time/position as well as
% useful values at each iteration
%
%  David Ufford
%  June 2009

dt1_i=Q0(7)%keep track of initial conditions for fitness calculations
dt2_i=Q0(8)

clear global next_swap;%ensure these are cleared between runs
clear global magnetState;
clear global motorState;

%%%% intialize variables
global magnetState;%0 = magnets off, %1 = magnet 1 on.  %2= magnet 2 on %3=both magnets on
global motorState;%= current value of torque being applied
global terminated_t;%method of passing the current state when control algorimthm
global terminated_Q;%  determines a ode45 termination is necessary
global doplot;

DQcheck=@(t,Q,flag)DiffEqMonitor(t,Q,flag,M,I,L,R); %monitoring function

%ode options.  Error tolerance has been tightened
options = odeset('RelTol',1e-9,'AbsTol',1e-9,'OutputFcn',DQcheck);

%doublecheck doplot is set
if(~exist('doplot') || length(doplot)<1)
    doplot=0;
end

t=0;  %current time
fixedEnd=1;  %start with Magnet '1' is on top
magnetState=1;  %and it is fixed to the wall
motorState=0; %start with motor off
terminated_t=[];
terminated_Q=[];


%precalc some stuff
m2r2=M(2)*R(2)^2;
Mass_constant=[I(1)+I(2)+M(1)*R(1)^2+M(2)*L(1)^2+m2r2,        m2r2+I(2);
    m2r2+I(2)       I(2)+m2r2];%constants added on to mass matrix
Mass_var=[2*M(2)*R(2)*L(1), M(2)*R(2)*L(1);
    M(2)*R(2)*L(1) 0];%constants multiplied by q for mass matrix
C_var=[-M(2)*R(2)*L(1)       -M(2)*R(2)*L(1);
    M(2)*R(2)*L(1)     0];
G_var=[g*(M(1)*R(1)+M(2)*L(1)) g*M(2)*R(2);
    0 g*M(2)*R(2)];

Q=Q0;
final_t=[];
final_Q=[];


%calculate step '0' values
step=1;
step_y(step)=Q(2);
step_fitness(step)=(dt1_i-Q(7))^2 + (dt2_i-Q(8))^2 + (Q(4))^2;
step_energy(step)=calcKEnergy(Q,M,I,L,R,g);

disp('Solving....');

while (step<=stepLimit+1)%loop while we are still within step

    terminated_t=-1;%reset these variables
    terminated_Q=[];

    %create function handle to DiffEqWrapper with correct parameters
    DiffEq=@(t,Q)DiffEqWrapper(t,Q,magnetState,Mass_constant,Mass_var,C_var,G_var, M,I,L,R,g);  %reduce number of parameters

    tspan=t:.001:t+1; %setup time span -> request intervals of .001
    %(these intervals impact reported and precision of controller,
    %     , not accuracy of integration)


    %%INTEGRATE
    [T, state]=ode45(DiffEq,tspan, Q,options); %--->GO!

    %%OBTAIN CORRECT FINAL STATE
    if(terminated_t>0) %integration was halted due to controller change,
        t=terminated_t;
        Q=terminated_Q;   %get values at termination

        ii = find(T==terminated_t);
        T(ii:end)=[];  %remove extraneous values past the termination point
        state(ii:end,:)=[];
    else
        t=T(end); %obtain final values for next pass through ode45
        Q=state(end,:);
    end

    final_t=[final_t; T]; %store values in array
    final_Q=[final_Q; state];


    %%%%CHECK FOR IMPACTS
    if(magnetState >0 && magnetState~=fixedEnd)
        %magnet needs to be swapped


        %NET1=calcEnergy(Q,M,I,L,R,g);% calculate pre-impact energys

        %%%%5.1 Reposition variables so that other end is the base x/y
        %         (flip system)
        [Q M I L R]=flipEnds(Q,M,I,L,R);

        %%%%5.2  Calculate post-impact velocties (impact at the new x/y)
        dq=Q(5:8);
        P=buildP(Q,M,I,L,R);%Build Projection Matrix
        dq=P*dq;  %...calculate post-impact velocities (post)=P*(pre)
        Q=[Q(1:4); dq]; %compile new state vector


        %NET=[NET1 NET2];

        %disp(['transition at' num2str(t)]);

        fixedEnd=magnetState; %swap the end that is attached to the magent

        %%%%INCREMENT STEP/iteration COUNT
        step=step+1;

        %%find when t1=0 (robot at vertical) so we can calculate fitness
        ii=find( abs(state(:,3))==min(abs(state(:,3))) ,1,'last');
        
        %calculate iteration/step values for stability
        step_energy(step-1)=calcKEnergy(state(ii,:),M,I,L,R,g); %kinetic energy at vertical
        step_fitness(step-1)=(state(ii,7)-dt1_i)^2 + (state(ii,8)-dt2_i)^2 + state(ii,4)^2;%fitness function
        step_y(step)=Q(2);
    end


end %loop until we reached the timeLimit for simulation
disp('Done!');


function P=buildP(Q,M,I,L,R)
%Builds the projection matrix used during impact

t1=Q(3);
t2=Q(4);

%sins/cosines
C1=cos(t1);
C12=cos(t1+t2);
C2=cos(t2);
S1=sin(t1);
S12=sin(t1+t2);
S2=sin(t2);

mrml=M(1)*R(1)+M(2)*L(1);

Mass=[M(1)+M(2)   0   (mrml)*C1+M(2)*R(2)*C12  M(2)*R(2)*C12;
    0       M(1)+M(2)   (mrml)*S1+M(2)*R(2)*S12  M(2)*R(2)*S12;
    (mrml)*C1+M(2)*R(2)*C12          (mrml)*S1+M(2)*R(2)*S12 ...
    I(1)+I(2)+M(1)*R(1)^2+M(2)*L(1)^2+M(2)*R(2)^2+2*M(2)*R(2)*L(1)*C2         M(2)*R(2)^2+M(2)*R(2)*L(1)*C2+I(2);
    M(2)*R(2)*C12       M(2)*R(2)*S12       M(2)*R(2)^2+M(2)*R(2)*L(1)*C2+I(2)      I(2)+M(2)*R(2)^2];

invM=inv(Mass);

J=[1 0 0 0;
    0 1 0 0];

P=eye(4)-invM*(J.')*inv(J*invM*(J.'))*J;
